@@ -40,6 +40,7 @@
 #define NULL 0
 #endif
 
+#include "misc.h"
 #include <speex/speex.h>
 #include <speex/speex_bits.h>
 #include <speex/speex_jitter.h>
@@ -58,7 +59,7 @@ void speex_jitter_init(SpeexJitter *jitter, void *decoder, int sampling_rate)
 
    jitter->dec = decoder;
    speex_decoder_ctl(decoder, SPEEX_GET_FRAME_SIZE, &jitter->frame_size);
-   jitter->frame_time = 1000*jitter->frame_size / sampling_rate;
+   jitter->frame_time = jitter->frame_size;
 
    speex_bits_init(&jitter->current_packet);
    jitter->valid_bits = 0;
@@ -73,6 +74,7 @@ void speex_jitter_init(SpeexJitter *jitter, void *decoder, int sampling_rate)
 
 void speex_jitter_destroy(SpeexJitter *jitter)
 {
+   speex_bits_destroy(&jitter->current_packet);
 }
 
 
@@ -139,7 +141,7 @@ void speex_jitter_put(SpeexJitter *jitter, char *packet, int len, int timestamp)
    /* Copy packet in buffer */
    if (len>SPEEX_JITTER_MAX_PACKET_SIZE)
       len=SPEEX_JITTER_MAX_PACKET_SIZE;
-   for (j=0;j<len;j++)
+   for (j=0;j<len/BYTES_PER_CHAR;j++)
       jitter->buf[i][j]=packet[j];
    jitter->timestamp[i]=timestamp;
    jitter->len[i]=len;
@@ -210,7 +212,7 @@ void speex_jitter_get(SpeexJitter *jitter, short *out, int *current_timestamp)
    {
       jitter->shortterm_margin[MAX_MARGIN-1] += jitter->shortterm_margin[MAX_MARGIN-2];
       jitter->longterm_margin[MAX_MARGIN-1] += jitter->longterm_margin[MAX_MARGIN-2];
-      for (i=MAX_MARGIN-2;i>=0;i--)
+      for (i=MAX_MARGIN-3;i>=0;i--)
       {
          jitter->shortterm_margin[i+1] = jitter->shortterm_margin[i];
          jitter->longterm_margin[i+1] = jitter->longterm_margin[i];         
