@@ -1,7 +1,9 @@
-/* Copyright (C) 2002 Jean-Marc Valin 
-   File: ltp.h
-   Long-Term Prediction functions
-
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file ltp.h
+   @brief Long-Term Prediction functions
+*/
+/*
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
@@ -33,14 +35,15 @@
 #include <speex/speex_bits.h>
 #include "misc.h"
 
-typedef struct ltp_params {
+/** LTP parameters. */
+typedef struct {
    const signed char *gain_cdbk;
    int     gain_bits;
    int     pitch_bits;
 } ltp_params;
 
 #ifdef FIXED_POINT
-#define gain_3tap_to_1tap(g) (ABS(g[1]) + (g[0]>0 ? g[0] : -SHR(g[0],1)) + (g[2]>0 ? g[2] : -SHR(g[2],1)))
+#define gain_3tap_to_1tap(g) (ABS(g[1]) + (g[0]>0 ? g[0] : -SHR16(g[0],1)) + (g[2]>0 ? g[2] : -SHR16(g[2],1)))
 #else
 #define gain_3tap_to_1tap(g) (ABS(g[1]) + (g[0]>0 ? g[0] : -.5*g[0]) + (g[2]>0 ? g[2] : -.5*g[2]))
 #endif
@@ -65,9 +68,10 @@ int   nsf,                      /* Number of samples in subframe */
 SpeexBits *bits,
 char *stack,
 spx_sig_t *exc2,
-spx_sig_t *r,
+spx_word16_t *r,
 int   complexity,
-int   cdbk_offset
+int   cdbk_offset,
+int plc_tuning
 );
 
 /*Unquantize adaptive codebook and update pitch contribution*/
@@ -105,9 +109,10 @@ int   nsf,                      /* Number of samples in subframe */
 SpeexBits *bits,
 char *stack,
 spx_sig_t *exc2,
-spx_sig_t *r,
+spx_word16_t *r,
 int complexity,
-int cdbk_offset
+int cdbk_offset,
+int plc_tuning
 );
 
 /** Unquantize forced pitch delay and gain */
