@@ -57,7 +57,7 @@ typedef void (*lsp_unquant_func)(spx_lsp_t *, int, SpeexBits *);
 /** Long-term predictor quantization */
 typedef int (*ltp_quant_func)(spx_sig_t *, spx_sig_t *, spx_coef_t *, spx_coef_t *, 
                               spx_coef_t *, spx_sig_t *, const void *, int, int, spx_word16_t, 
-                              int, int, SpeexBits*, char *, spx_sig_t *, spx_sig_t *, int, int);
+                              int, int, SpeexBits*, char *, spx_sig_t *, spx_word16_t *, int, int, int);
 
 /** Long-term un-quantize */
 typedef void (*ltp_unquant_func)(spx_sig_t *, int, int, spx_word16_t, const void *, int, int *,
@@ -66,7 +66,7 @@ typedef void (*ltp_unquant_func)(spx_sig_t *, int, int, spx_word16_t, const void
 
 /** Innovation quantization function */
 typedef void (*innovation_quant_func)(spx_sig_t *, spx_coef_t *, spx_coef_t *, spx_coef_t *, const void *, int, int, 
-                                      spx_sig_t *, spx_sig_t *, SpeexBits *, char *, int, int);
+                                      spx_sig_t *, spx_word16_t *, SpeexBits *, char *, int, int);
 
 /** Innovation unquantization function */
 typedef void (*innovation_unquant_func)(spx_sig_t *, const void *, int, SpeexBits*, char *);
@@ -111,7 +111,7 @@ typedef struct SpeexNBMode {
    spx_word16_t gamma1;    /**< Perceptual filter parameter #1 */
    spx_word16_t gamma2;    /**< Perceptual filter parameter #2 */
    float   lag_factor;     /**< Lag-windowing parameter */
-   float   lpc_floor;      /**< Noise floor for LPC analysis */
+   spx_word16_t   lpc_floor;      /**< Noise floor for LPC analysis */
 
 #ifdef EPIC_48K
    int     lbr48k;         /**< 1 for the special 4.8 kbps mode */
@@ -133,7 +133,7 @@ typedef struct SpeexSBMode {
    spx_word16_t gamma1;   /**< Perceptual filter parameter #1 */
    spx_word16_t gamma2;   /**< Perceptual filter parameter #1 */
    float   lag_factor;    /**< Lag-windowing parameter */
-   float   lpc_floor;     /**< Noise floor for LPC analysis */
+   spx_word16_t   lpc_floor;     /**< Noise floor for LPC analysis */
    float   folding_gain;
 
    const SpeexSubmode *submodes[SB_SUBMODES]; /**< Sub-mode data for the mode */
