@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2003 Jean-Marc Valin 
+/* Copyright (C) 2002-2006 Jean-Marc Valin 
    File: speexenc.c
 
    Redistribution and use in source and binary forms, with or without
@@ -147,13 +147,13 @@ static int read_samples(FILE *fin,int frame_size, int bits, int channels, int ls
 void version()
 {
    printf ("speexenc (Speex encoder) version " SPEEX_VERSION " (compiled " __DATE__ ")\n");
-   printf ("Copyright (C) 2002-2005 Jean-Marc Valin\n");
+   printf ("Copyright (C) 2002-2006 Jean-Marc Valin\n");
 }
 
 void version_short()
 {
    printf ("speexenc version " SPEEX_VERSION "\n");
-   printf ("Copyright (C) 2002-2005 Jean-Marc Valin\n");
+   printf ("Copyright (C) 2002-2006 Jean-Marc Valin\n");
 }
 
 void usage()
@@ -477,6 +477,7 @@ int main(int argc, char **argv)
       rate=8000;
    } else if (modeID!=-1 && rate)
    {
+      mode = speex_lib_get_mode (modeID);
       if (rate>48000)
       {
          fprintf (stderr, "Error: sampling rate too high: %d Hz, try down-sampling\n", rate);
@@ -536,7 +537,8 @@ int main(int argc, char **argv)
       if (rate!=8000 && rate!=16000 && rate!=32000)
          fprintf (stderr, "Warning: Speex is only optimized for 8, 16 and 32 kHz. It will still work at %d Hz but your mileage may vary\n", rate); 
 
-   mode = speex_lib_get_mode (modeID);
+   if (!mode)
+      mode = speex_lib_get_mode (modeID);
 
    speex_init_header(&header, rate, 1, mode);
    header.frames_per_packet=nframes;
@@ -628,8 +630,9 @@ int main(int argc, char **argv)
 
    /*Write header*/
    {
-
-      op.packet = (unsigned char *)speex_header_to_packet(&header, (int*)&(op.bytes));
+      int packet_size;
+      op.packet = (unsigned char *)speex_header_to_packet(&header, &packet_size);
+      op.bytes = packet_size;
       op.b_o_s = 1;
       op.e_o_s = 0;
       op.granulepos = 0;
